function [y,s]=jm_infer_1step(TH,x,X,Y,Linit,Lmode,Ltrans,loss_type)
%#codegen
% Infer one-step ahead prediction of yt given past data and modes.
%
% [y,s]=jm_infer_1step(TH,x,X,Y,Linit,Lmode,Ltrans,loss_type)
%
% Input arguments:
% ----------------
% TH: model parameters   [nx-by-K-by-ny array] for loss_type=1,2
%                        or [nx-by-K array] for loss_type=3
% x: x(t)                [nx-by-1 array]
% X: x(1),...,x(t-1)     [(t-1)-by-nx array]
% Y: y(1),...,y(t-1)     [(t-1)-by-ny array]
% Linit: penalty on initial mode s(0) [K-by-1 array]
% Lmode: penalty on mode s(k), k>0 [K-by-1 array]
% Ltrans: penalty on transitions [K-by-K array]
% loss_type:      1 = least squares ||y-theta*x||_2^2          [only ny=1 supported]
%                 2 = binary classification max(0,1-y*theta*x) [only ny=1 supported]
%                 3 = ||y-theta||_2^2
%
% Output arguments:
% ----------------
% y: yhat(t)
% s: shat(t)
%
% (C) 2017 A. Bemporad, July 25, 2017
%          Revised by A. Bemporad, October 2, 2017

inference_flag=1;
ny=size(Y,2);

% Estimate s(t-1)
S=jm_dp_mex([X;x'],[Y;zeros(1,ny)],TH,Linit,Lmode,Ltrans,inference_flag,loss_type);
s=S(end);

y=zeros(ny,1);
switch loss_type
    case 1
        for i=1:ny
            y(i)=TH(:,s,i)'*x;
        end
    case 2
        for i=1:ny
            y(i)=sign(TH(:,s,i)'*x);
        end
    case 3
        y=TH(:,s);
end