function [loss,Yhat]=jm_loss(X,Y,S,TH,loss_type,tau_PWA,c)
%#codegen
% Compute loss function and inferred outputs associated with model TH, 
% mode sequence S, data X,Y.
%
% [loss,Yhat]=jm_loss(X,Y,S,TH,loss_type)
%
% Input arguments:
% ----------------
% X: x(1),...,x(T)       [T-by-nx array]
% Y: y(1),...,y(T)       [T-by-ny array]
% S: s(0),...,s(T)       mode sequence [(T+1)-by-1 array] (s(0) is not used)
% TH: model parameters   [nx-by-K-by-ny array] for loss_type=1,2
%                        or [nx-by-K array] for loss_type=3
% loss_type:      1 = least squares ||y-theta*x||_2^2
%                 2 = binary classification max(0,1-y*theta*x)
%                     (or sum_j(max(0,1-y(j)*theta_j*x)) in case of dim(y)>1)
%                 3 = ||y-theta||_2^2
%                 4 = PWA regression over Voronoi partition. The cost is
%                     ||y-theta_{y,s}*x||_2^2+tau_PWA*||x-theta_{x,s}||_2^2 
% tau_PWA: [scalar] hyper-parameter (only required for loss_type=4)
% c: cluster centroids (only for PWA regression) [K-by-n matrix]
%
% Output arguments:
% ----------------
% loss: loss function
% Yhat: estimated outputs
%
% (C) 2017 A. Bemporad, October 2, 2017
%     Revised by D. Piga, October 15, 2017 (PWA models)
%     Revised by A. Bemporad, October 21, 2017

[T,ny]=size(Y);
loss=0;
Yhat=zeros(T,ny);

for i=1:T
    xi=X(i,:);
    yi=Y(i,:);
    k=S(i+1);
    switch loss_type
        case {1,4}
            for t=1:ny
                Yhat(i,t)=xi*TH(:,k,t);
            end
            loss=sum((yi-Yhat(i,:)).^2);
        case 2
            loss=0;
            for t=1:ny
                aux=xi*TH(:,k,t);
                Yhat(i,t)=sign(aux); % we should assign 0 to +1 or -1 here
                loss=loss+max(1-yi(t)*aux,0);
            end
        case 3
            Yhat(i,:)=TH(:,k);
            loss=sum((yi-Yhat(i,:)).^2);
    end
end