function [Ltrans2,Linit2]=jm_mode_loss(P,p,tau,tau0)
%#codegen
% Update mode loss function based on empirical transition probabilities.
%
% [Ltrans2,Linit2]=jm_mode_loss(P,p,tau,tau0)
%
% P and p are usually obtained by running function jm_trans_prob on an
% estimated mode sequence.
%
% Input arguments:
% ----------------
% P: transition probabilities [K-by-K array]. 
% p: initial mode probabilities [K-by-1 array]
% tau: Ltrans2 is defined so that sum_i(Ltrans2(i,:))=tau, 
%      where the sum is on the indices i such that P(i,h)>0 for some h
% tau0: Linit2 is defined so that sum_i(Linit2(i,:))=tau0,
%      where the sum is on the indices i such that p(i)>0
%
% Output arguments:
% ----------------
% Ltrans: updated transition loss
% Linit: updated initial mode loss
%
% (C) 2017 by A. Bemporad, October 2, 2017

bignum=1e8;
smallnum=0;

K=numel(tau);
Linit2=zeros(K,1);
isLz=false(K,1);

for i=1:K
    if p(i)>smallnum
        Linit2(i)=-log(p(i));
        isLz(i)=true;
    end
end
Linit2=Linit2/sum(Linit2)*tau0; % preserves sum of nonzero terms
Linit2(~isLz)=bignum;

Ltrans2=zeros(K,K);
for i=1:K
    isLz=false(K,1);
    for h=1:K
        if P(i,h)>smallnum
            Ltrans2(i,h)=-log(P(i,h));
            isLz(h)=true;
        end
    end
    Ltrans2(i,:)=Ltrans2(i,:)/sum(Ltrans2(i,:))*tau(i); % preserves sum of nonzero terms
    Ltrans2(i,~isLz)=bignum;
end
