function [P,p0]=jm_trans_prob(S,K)
%#codegen
% Compute empirical transition probabilities from discrete sequence.
%
% [P,p0]=jm_trans_prob(S,K)
%
% Input arguments:
% ----------------
% S: s(0),...,s(T)       [(T+1)-by-1 array]
% K: number of modes     [scalar]
%
% Output arguments:
% ----------------
% P:  transition probability matrix [K-by-K array]
%     P(i,j) = probability of switching from s(t-1)=j to s(t)=i.
% p0: probability of initial mode [K-by-1 array]
%     p0(i) = probability that s(0)=i, computed assuming that each s(t) in
%     the sequence is a potential initial mode.
%
% (C) 2017 A. Bemporad, July 20-26, 2017
%          Revised by A. Bemporad, October 2, 2017

N=numel(S);
P=zeros(K,K);
p0=zeros(K,1);
for j=1:K
    Nj=sum(S(1:N-1)==j);
    for i=1:K
        Nij=sum(S(2:end)==i & S(1:end-1)==j);
        P(j,i)=Nij/Nj;
    end
    Nj=Nj+(S(N)==j);
    p0(j)=Nj/(N+1);
end